#include "Enemy.h"

#define MOVE_POSITION_COUNT_ 8
#define NEXT_MOVE_CHOICE_COUNT_ 4

void initEnemies()
{
	u16 i = 0;

	for (i = 0; i < mapEnemyCount; i += 3)
	{
		// Dark mines use state information for something else and don't need health due to killing power.
		if ((mapEnemies[i + 2] & ENEMY_GFX_RANGE) != ENEMY_DARK_MINE)
		{
			// Add health bits to state, bitshift these left when the enemy is attacked.
			mapEnemies[i + 2] |= ENEMY_HEALTH_FLAG;
		}
	}
}

void drawEnemies(u8 xScroll, u8 yScroll)
{
	if (enemyRedraw)
	{
		u16 i = 0;
		u8 cell = 0;

		// Draw each of the enemies where I determine if the enemies are actually on screen.
		for (i = 0; i < mapEnemyCount; i += 3)
		{
			cell = MAPA(mapEnemies[i], mapEnemies[i + 1]);

			// Enemy of type 0 is dead.
			if (mapEnemies[i + 2])
			{
				drawEnemy(mapEnemies[i] - xScroll, mapEnemies[i + 1] - yScroll, mapEnemies[i + 2], cell);
			}
		}

		enemyRedraw = FALSE;
	}
}

void drawEnemy(i8 x, i8 y, u8 enemyType, u8 cell)
{
	// Make sure the enemy is in bounds on the x axis.
	if (x >= 0 && x < SCREEN_TILES_X)
	{
		// Make sure the enemy is in bounds on the y axis.
		if (y >= 0 && y < SCREEN_TILES_Y)
		{
			u8* mapMemory = cpct_getScreenPtr(CPCT_VMEM_START, xPositions[x], yPositions[y]);

			// Only draw the enemy if the cell is lit.
			if (cell & CELL_LIGHT_FLAG)
			{
				// Dark mine is not visible if the cell is lit.
				if ((enemyType & ENEMY_GFX_RANGE) != ENEMY_DARK_MINE)
				{
					// Draw an enemy from the graphicset.	
					cpct_drawSprite(enemyGraphics[(enemyType & ENEMY_GFX_RANGE) - 1], mapMemory, TILE_WIDTH_BYTES, TILE_HEIGHT);
				}
			}
			else
			{
				// Dark mine is only visible in the dark when has no state information (the top 5 bits must be empty).
				if (enemyType == ENEMY_DARK_MINE)
				{
					cpct_drawSprite(enemyGraphics[(enemyType & ENEMY_GFX_RANGE) - 1], mapMemory, TILE_WIDTH_BYTES, TILE_HEIGHT);
				}

				// TODO consider drawing something if the other enemy types are not visible.
			}
		}
	}
}

void logicEnemies()
{
	// Run through the logic for all the enemies if it is the enemies turn.
	if (gameTurnState == ENEMY_TURN)
	{
		u16 i = 0;
		u8 cell = 0;

		// Enemy direcions are:
		// 0 = RIGHT ( 1, 0)
		// 1 = DOWN  ( 0, 1)
		// 2 = LEFT  (-1, 0)
		// 3 = UP    ( 0,-1)
		i8 directionX[DIRECTION_COUNT] = { 1, 0, -1, 0 };
		i8 directionY[DIRECTION_COUNT] = { 0, 1, 0, -1 };

		// Direction orders (each set of directions has a slightly different order).
		// 0, 1, 3, 2
		// 1, 2, 0, 3
		// 2, 3, 1, 0
		// 3, 0, 2, 1
		u8 directionOrder[DIRECTION_COUNT] = { 0, 1, 3, 2 };		

		// Run through the logic for all the enemies (currently includes off screen as well).
		for (i = 0; i < mapEnemyCount; i += 3)
		{
			u8 x = mapEnemies[i];
			u8 y = mapEnemies[i + 1];
			u8* enemyType = &mapEnemies[i + 2];
			
			// Retrieve the values from the direcion bits (2^4,2^3) in the enemy state and bit shift 3 to put them in 2^1,2^0.
			u8 enemyDirection = (*enemyType & ENEMY_DIRECTION_BITS) >> 3;

			cell = MAPA(x, y);

			// Run enemy attack before move.
			if ((*enemyType & ENEMY_GFX_RANGE) != ENEMY_DARK_MINE)
			{
				if (enemyAttackLogic(x, y, *enemyType))
				{
					// The attack occured so the turn ends and the enemy doesn't move.
					continue;
				}				
			}

			// Logic for the dark mine.
			if ((*enemyType & ENEMY_GFX_RANGE) == ENEMY_DARK_MINE)
			{
				// Check that the cell is currently lit.
				if (cell & CELL_LIGHT_FLAG)
				{
					// Reset enemy state to just the enemy type (this resets the upper 5 bits to 0 making 0b00000100.					
					// This is what the 4 is for.

					// Setting bit 3 to indicate the mine is currently lit.
					// This indicator will be used to determine when the mine can be made visible again.
					// While this indicator is set the mine will be invisible in the dark.
					// This is what the 8 is for.

					// 4 + 8 = 12
					*enemyType = 12;
				}
				else
				{
					// Check if additional state information has been set.
					// This state information determines how many turns since the mine was lit.
					// 248 is the top 5 bits 0b11111000.
					if (*enemyType & 248)
					{
						// Bit shift the top 5 bits to the left and then add the enemy type back.
						// Once the indicator falls off the mine is back to normal (invisible in the dark).
						*enemyType = ((*enemyType & 248) << 1) + 4;
					}
				}
			}
			else if ((*enemyType & ENEMY_GFX_RANGE) == ENEMY_GUARD)
			{
				// Positions (x,y) in an around an object.
				i8 moveX[MOVE_POSITION_COUNT_] = { -1, 0, 1, 1, 1, 0,-1,-1 };
				i8 moveY[MOVE_POSITION_COUNT_] = { -1,-1,-1, 0, 1, 1, 1, 0 };

				// Possible relative move choices within the move arrays when an enemy is moving around an object.
				i8 nextMoveChoices[NEXT_MOVE_CHOICE_COUNT_] = { 1, -1, 2, -2 };

				u8 j = 0;

				// Where the enemy is in the move arrays.
				u8 currentAiMove = 255;

				// The object that is being guarded.
				u8 objectX = 255;
				u8 objectY = 255;

				u8 testX = 0;
				u8 testY = 0;
				
				// Determines the position of an item or interactable (object) around the enemy.
				for (j = 0; j < MOVE_POSITION_COUNT_; ++j)
				{
					u8 testX = x + moveX[j];
					u8 testY = y + moveY[j];

					// Make sure that the possible move positions are within the map bounds.
					if (testX < MAP_TILES_X && testY < MAP_TILES_Y)
					{
						if (MAPA(testX, testY) & CELL_INTERACTABLE_OR_ITEM_FLAG)
						{
							objectX = testX;
							objectY = testY;

							// Found the object so break from the loop (after the first object).
							break;
						}
					}
				}

				// Test if an object has been found near the enemy (we don't need to test both objectX and objectY).
				if (objectX != 255)
				{
					u8 nextMove = 255;
					u8 testX = 0;
					u8 testY = 0;

					// Determines where the enemy currently resides around the item or interactable (object).
					for (j = 0; j < MOVE_POSITION_COUNT_; ++j)
					{
						testX = objectX + moveX[j];
						testY = objectY + moveY[j];

						if (testX == x && testY == y)
						{
							currentAiMove = j;
							
							// Having found the position of the enemy around the object we break out of the loop.
							break;
						}
					}

					// Determine the next move for the enemy and that it is within 0 to 7.
					for (j = 0; j < NEXT_MOVE_CHOICE_COUNT_; ++j)
					{
						u8 testMove = currentAiMove + nextMoveChoices[j];
						
						// Determine that the next move is inbound.
						if (testMove >= MOVE_POSITION_COUNT_)
						{
							// testMove = 8, nextMove = 8 - 8 = 0, testMove = 9, nextMove = 9 - 8 = 1
							nextMove = testMove - MOVE_POSITION_COUNT_;
						}
						else if (testMove < 0)
						{
							// testMove = -1, nextMove = -1 + 8 = 7, testMove = -2, nextMove = -2 + 8 = 6
							nextMove = testMove + MOVE_POSITION_COUNT_;
						}
						else
						{
							nextMove = testMove;
						}

						// Test that the next position around the object is empty.
						testX = objectX + moveX[nextMove];
						testY = objectY + moveY[nextMove];

						// Check that the next position is within the map bounds and that the next cell is empty.
						if ((testX < MAP_TILES_X && testY < MAP_TILES_Y) && !(MAPA(testX, testY) & CELL_GFX_RANGE))
						{
							// Cell is empty so we can move the enemy now.
							mapEnemies[i] = testX;
							mapEnemies[i + 1] = testY;

							// Set the enemy indicator interactable on the new cell.
							MAPA(testX, testY) = MAPA(testX, testY) & CELL_LIGHT_FLAG | CELL_INTERACTABLE_FLAG | INTERACTABLE_ENEMY_INDICATOR;

							// Remove the enemy indicator and interactable flags but add the light flag if it is there (old cell).
							MAPA(x, y) = MAPA(x, y) & CELL_LIGHT_FLAG;

							// Jump out of the loop once the enemy has moved.
							break;
						}
					}

					// TODO consider what happens if all the next move positions are unavailable and enemy is effectively stuck.
					// Either do nothing (current action) or move it somewhere else randomly.
				}
				// TODO handle moving off (iratically?) after no object has been found.
			}
			else if ((*enemyType & ENEMY_GFX_RANGE) == ENEMY_LIGHT_ATTACKER)
			{
				// TODO update the name of the enemy type to aimless or erratic sometime probably in task #2279.
				u8 j = 0;
				u8 testX = 255;
				u8 testY = 255;

				for (j = 0; j < DIRECTION_COUNT; ++j)
				{
					// Determine the direction value based on the order.
					// The direction value is adjusted using the original enemy direction to get the correct direction order set.
					u8 currentDirection = directionOrder[j] + enemyDirection;

					if (currentDirection >= DIRECTION_COUNT)
					{
						// 4 - 4 = 0
						// 5 - 4 = 1
						// 6 - 4 = 2
						currentDirection -= DIRECTION_COUNT;
					}

					// Add direction to testX and testY.
					testX = x + directionX[currentDirection];
					testY = y + directionY[currentDirection];

					// Make sure that the possible move positions are within the map bounds.
					if (testX < MAP_TILES_X && testY < MAP_TILES_Y)
					{
						// Make sure the cell at (testX, testY) is empty.
						if (!(MAPA(testX, testY) & CELL_GFX_RANGE))
						{
							// Cell is empty so we can move the enemy now.
							mapEnemies[i] = testX;
							mapEnemies[i + 1] = testY;
							
							// 231 = 0b11100111 (this will clear the direction bits).
							*enemyType &= 231;

							// Update enemy state with current direction (bitshift currentDirection to be in the right place).
							*enemyType |= (currentDirection << 3);

							// Set the enemy indicator interactable on the new cell.
							MAPA(testX, testY) = MAPA(testX, testY) & CELL_LIGHT_FLAG | CELL_INTERACTABLE_FLAG | INTERACTABLE_ENEMY_INDICATOR;

							// Remove the enemy indicator and interactable flags but add the light flag if it is there (old cell).
							MAPA(x, y) = MAPA(x, y) & CELL_LIGHT_FLAG;

							// Break out of the loop as enemy has now moved.
							break;
						}
					}
				}
			}
			else if ((*enemyType & ENEMY_GFX_RANGE) == ENEMY_MOTH)
			{
				u8 j = 0;
				u8 testX = 255;
				u8 testY = 255;

				// Set light(x,y) to enemy(x,y) as the source position to determine the nearest light.
				u8 lightX = x;
				u8 lightY = y;

				// Determine the nearest active light source.
 				if (nearestLight(&lightX, &lightY))
				{
					u8 xDifference = (u8) absoluteValue(difference(x, lightX));
					u8 yDifference = (u8) absoluteValue(difference(y, lightY));
					u8 chosenDirection = 255;

					// Check if the moth is on top of the light source.
					if (xDifference == 0 && yDifference == 0)
					{
						// The moth does not need to move if it is already on top of the light source.
						continue;
					}

					// Chose the direction based on which axis is currently closer.
					// If xDifference is 0 we use y.
					// If yDifference is 0 we use x.
					if ((xDifference < yDifference && xDifference != 0) || yDifference == 0)
					{
						// Determine which direction (left or right) is closer.
						if (x < lightX)
						{
							// Right
							chosenDirection = ENEMY_DIRECTION_RIGHT_;
						}
						else
						{
							// Left
							chosenDirection = ENEMY_DIRECTION_LEFT_;							
						}
					}
					else
					{
						// Determine which direction (up or down) is closer.
						if (y < lightY)
						{
							// Down
							chosenDirection = ENEMY_DIRECTION_DOWN_;							
						}
						else
						{
							// Up
							chosenDirection = ENEMY_DIRECTION_UP_;
						}
					}

					// The enemy has previously been blocked and still wants to go in that direction.
					if (chosenDirection == enemyDirection && (*enemyType & ENEMY_BLOCKED_FLAG))
					{
						// Chose a new direction with the previous direction the last one in the direction order.

						// Adding two will invert the direction.
						chosenDirection += 2;

						// Make sure the direction is valid (under 4).
						if (chosenDirection >= DIRECTION_COUNT)
						{
							// 4 - 4 = 0
							// 5 - 4 = 1
							// 6 - 4 = 2
							chosenDirection -= DIRECTION_COUNT;
						}
					}
					else if (chosenDirection != enemyDirection && !(*enemyType & ENEMY_BLOCKED_FLAG))
					{
						// Not blocked and not the same direction as previous.
						u8 tempDirection = chosenDirection;
						
						// Adding two will invert the direction.
						tempDirection += 2;

						// Make sure the direction is valid (under 4).
						if (tempDirection >= DIRECTION_COUNT)
						{
							// 4 - 4 = 0
							// 5 - 4 = 1
							// 6 - 4 = 2
							tempDirection -= DIRECTION_COUNT;
						}

						// Check to see if the direction has been inverted.
						if (tempDirection == enemyDirection)
						{
							// Having inverted the previous direction we now want to revert that so the enemy doesn't oscillate between two positions.
							chosenDirection = enemyDirection;
						}
					}

					for (j = 0; j < DIRECTION_COUNT; ++j)
					{
						// Determine the direction value based on the order.
						// The direction value is adjusted using the chosen direction to get the correct direction order set.
						u8 currentDirection = directionOrder[j] + chosenDirection;

						u8 testX = 255;
						u8 testY = 255;

						// Make sure the direction is valid (under 4).
						if (currentDirection >= DIRECTION_COUNT)
						{
							// 4 - 4 = 0
							// 5 - 4 = 1
							// 6 - 4 = 2
							currentDirection -= DIRECTION_COUNT;
						}

						// Add direction to testX and testY.
						testX = x + directionX[currentDirection];
						testY = y + directionY[currentDirection];

						// Make sure that the possible move positions are within the map bounds.
						if (testX < MAP_TILES_X && testY < MAP_TILES_Y)
						{
							// Make sure the cell at (testX, testY) is empty or is a light source sprite.
							if (!(MAPA(testX, testY) & CELL_GFX_RANGE) || ((MAPA(testX, testY) & CELL_GFX_RANGE) == (TILE_LIGHT_SOURCE)))
							{
								// Cell is empty so we can move the enemy now.
								mapEnemies[i] = testX;
								mapEnemies[i + 1] = testY;

								// Player has been blocked in all directions but the incoming direction so we are turning around.
								if (j == 3)
								{
									// Direction bits are not changed.

									// Set the enemy indicator interactable on the new cell, set blocked flag.
									MAPA(testX, testY) = MAPA(testX, testY) & CELL_LIGHT_FLAG | CELL_INTERACTABLE_FLAG | INTERACTABLE_ENEMY_INDICATOR | ENEMY_BLOCKED_FLAG;
								}
								else
								{
									// 231 = 0b11100111 (this will clear the direction bits).
									*enemyType &= 231;

									// Update enemy state with current direction (bitshift currentDirection to be in the right place).
									*enemyType |= (currentDirection << 3);

									// Set the enemy indicator interactable on the new cell. (block flag is not set)
									MAPA(testX, testY) = MAPA(testX, testY) & CELL_LIGHT_FLAG | CELL_INTERACTABLE_FLAG | INTERACTABLE_ENEMY_INDICATOR;
								}

								// Remove the enemy indicator and interactable flags but add the light flag if it is there (old cell).
								MAPA(x, y) = MAPA(x, y) & CELL_LIGHT_FLAG;

								// Break out of the loop as enemy has now moved.
								break;
							}
						}
					}
				}
			}
		}

		// Force a map redraw in case any of the enemies have moved on screen.
		mapRedraw = TRUE;

		// Move on to boss logic after enemy logic completed.
		gameTurnState = BOSS_TURN;
	}
}

u8 enemyAttackLogic(u8 x, u8 y, u8 enemyType)
{
	// Standard enemy types.
	i8 xDifference = 0;
	i8 yDifference = 0;

	xDifference = difference(player.object.position.x, x);
	yDifference = difference(player.object.position.y, y);

	// Check that the values are within one so is reachable by the enemy.
	if (xDifference > -2 && xDifference < 2 && yDifference > -2 && yDifference < 2)
	{
		// For now I am removing the wait for player attack as I don't think it works with the moth ai.
		//if ((enemyType & ENEMY_GFX_RANGE) == ENEMY_MOTH)
		//{
		//	// The enemy has been previously attacked.
		//	if (!(enemyType & ENEMY_HEALTH_FLAG))
		//	{
		//		// Attack the player only if previously have been attacked. (Counter attack only.)
		//		attackPlayer();

		//		// Attack has occured so enemy turn will end.
		//		return 1;
		//	}
		//}
		//else
		//{

		// Make sure that a dead enemy can't attack the player.
		if (enemyType)
		{
			// Attack the player.
			attackPlayer();

			// Attack has occured so enemy turn will end.
			return 1;
		}
			
		//}
	}

	// No attack occured so enemy turn continue.
	return 0;
}

void attackPlayer()
{
	// Check the player has health before attacking the player.
	if (player.health)
	{
		--player.health;
		hudRedraw = TRUE;
	}
}

u8 enemyScore(u8 type)
{
    switch(type)
    {
        case ENEMY_GUARD:
			return 10;

		case ENEMY_LIGHT_ATTACKER:
        case ENEMY_MOTH:
			return 20;

        default:
            //everything else is 100 (after the extra 0 is displayed) for now
            return 10;
    }
}