#ifndef GAMESTATE_H
#define GAMESTATE_H

#include <cpctelera.h>
#include "Difficulty.h"
#include "GameTypes.h"
#include "Lighting.h"

// The score for the whole game session.
extern u16 gameScore;

// The score for the current level.
extern u16 levelScore;

// The number of the level the player is on.
extern u8 currentLevel;

// Whether the player has ended the level (alive or dead).
extern u8 levelEnded;

// X position for where the player has been teleported to.
extern u8 playerTeleportingX;

// Y position for where the player has been teleported to.
extern u8 playerTeleportingY;

// Determines if the player is exiting the game.
extern u8 playerExiting;

// X position for where the player was before entering a cave entrance.
extern u8 preLevelTransitionPositionX;

// Y position for where the player was before entering a cave entrance.
extern u8 preLevelTransitionPositionY;

// Player steps used for oil fuel depletion.
extern u8 oilPlayerSteps;

// Player steps used for battery fuel depletion.
extern u8 batteryPlayerSteps;

//resets state values that are specific to the game session
void resetGameSessionState();

// Resets state values that are specific to the level session.
void resetLevelSessionState();

// Updates the game session with values from the level session.
void updateGameSessionState();

// Update the fuel based on the number of steps fitzroy has moved.
void updateFuelState();

// Determines if there is water on the level to animate.
extern u8 waterLevel;

// Determines if the player or enemies are currently having their turn.
extern u8 gameTurnState;

// Floor sprite that is drawn on the map.
extern u8* G_tile_floor_sprite;

// The amount of oil fuel the player is current carrying across the game session.
extern u8 oilFuel;

// The amount of battery fuel the player is currently carrying across the game session.
extern u8 batteryFuel;

// The amount of oil fuel the player is current carrying across the level session.
extern u8 levelOilFuel;

// The amount of battery fuel the player is currently carrying across the level session.
extern u8 levelBatteryFuel;

// The chosen move indicator for the player to use.
extern u8* G_move_indicator;

// The value to set a cell on the player moving off it or something being cleared off the cell such as a door.
extern u8 postMoveCell;

// Indicates which caves have been completed in the current game session.
extern u8 caveComplete[LEVEL_COUNT];

// The number of arrows that the player is carrying in the game session.
extern u8 arrowCount;

// The number of arrows that the player is carrying in the level session.
extern u8 levelArrowCount;

// The number of remaining moves that the player can move in the dark before dieing.
extern u8 darkMoveCount;

#endif