#include "Hud.h"

#define HUD_X_POSITION_ 0
#define HUD_X_POSITION_TWO_ 64
#define HUD_Y_POSITION_ 192
#define HUD_WIDTH_ONE_ 64
#define HUD_WIDTH_TWO_ 16
#define HUD_HEIGHT_ 8
#define HUD_ITEM_WIDTH_BYTES_ 3
#define HUD_ITEM_THIN_WIDTH_BYTES_ 2

#define HUD_SCORE_START_ 0
#define DARKMOVES_START_ 13
#define HEALTH_START_ 19
#define EQUIPPED_ITEM_LEFT_START_ 27
#define LIGHT_OIL_START_ 34
#define LIGHT_BATTERY_START_ 41
#define ARROW_START_ 49
#define MEAT_START_ 56

#define RED_KEY_START_ 64

u8 hudRedraw;

void initHud()
{
    hudRedraw = TRUE;
}

void drawHud()
{
    if(hudRedraw)
    {
        u8* hudVideoMemory;
        u8 hudText[6];
		u16 combinedScore = gameScore + levelScore;
		u8 extraScoreDigitGap = 0;

		// Testing
		/*levelOilFuel = 99;
		levelScore = 65356;
		levelBatteryFuel = 99;
		player.meat = 99;
		player.health = 99;
		levelArrowCount = 99;
		darkMoveCount = 22;
		INVENTORY_LINE_1 = 255;
		INVENTORY_LINE_2 = 255;*/

        //draw the hud strip on the background in two parts
		hudVideoMemory = cpct_getScreenPtr(CPCT_VMEM_START, HUD_X_POSITION_, HUD_Y_POSITION_);
		cpct_drawSolidBox(hudVideoMemory, cpct_px2byteM0(4, 4), HUD_WIDTH_ONE_, HUD_HEIGHT_);

		hudVideoMemory = cpct_getScreenPtr(CPCT_VMEM_START, HUD_X_POSITION_TWO_, HUD_Y_POSITION_);
		cpct_drawSolidBox(hudVideoMemory, cpct_px2byteM0(4, 4), HUD_WIDTH_TWO_, HUD_HEIGHT_);

		// Adjust the position for the extra 0 digit.
		if (combinedScore > 9)
		{
			extraScoreDigitGap = 4;
		}
		if (combinedScore > 99)
		{
			extraScoreDigitGap = 6;
		}
		else if (combinedScore > 999)
		{
			extraScoreDigitGap = 8;
		}
		else if (combinedScore > 9999)
		{
			extraScoreDigitGap = 10;
		}

		// Last score digit so the score looks more impressive without overflowing from u16.
		drawString(HUD_SCORE_START_ + extraScoreDigitGap, HUD_Y_POSITION_, "0");

		// Display game combine game score and level score.
		convertIntToChar(combinedScore, hudText);
		drawString(HUD_SCORE_START_, HUD_Y_POSITION_, hudText);

		// Display dark move count.
        hudVideoMemory = cpct_getScreenPtr(CPCT_VMEM_START, DARKMOVES_START_, HUD_Y_POSITION_);
        cpct_drawSprite(G_hud_dark_moves, hudVideoMemory, HUD_ITEM_THIN_WIDTH_BYTES_, HUD_HEIGHT_);
		convertIntToChar(darkMoveCount, hudText);
        drawString(DARKMOVES_START_ + HUD_ITEM_THIN_WIDTH_BYTES_, HUD_Y_POSITION_, hudText);
		
        //player health
        hudVideoMemory = cpct_getScreenPtr(CPCT_VMEM_START, HEALTH_START_, HUD_Y_POSITION_);
        cpct_drawSprite(G_hud_health, hudVideoMemory, HUD_ITEM_WIDTH_BYTES_, HUD_HEIGHT_);   
        convertIntToChar(player.health, hudText);
        drawString(HEALTH_START_ + HUD_ITEM_WIDTH_BYTES_, HUD_Y_POSITION_, hudText);	

		// Player equipped items.
		hudVideoMemory = cpct_getScreenPtr(CPCT_VMEM_START, EQUIPPED_ITEM_LEFT_START_, HUD_Y_POSITION_);
		drawEquippedItems(hudVideoMemory);

		// Light fuel.
		hudVideoMemory = cpct_getScreenPtr(CPCT_VMEM_START, LIGHT_OIL_START_, HUD_Y_POSITION_);
		cpct_drawSprite(G_hud_lamp_oil, hudVideoMemory, HUD_ITEM_WIDTH_BYTES_, HUD_HEIGHT_);
		convertIntToChar(levelOilFuel, hudText);
        drawString(LIGHT_OIL_START_ + HUD_ITEM_WIDTH_BYTES_, HUD_Y_POSITION_, hudText);	

		hudVideoMemory = cpct_getScreenPtr(CPCT_VMEM_START, LIGHT_BATTERY_START_, HUD_Y_POSITION_);
		cpct_drawSprite(G_hud_battery, hudVideoMemory, HUD_ITEM_WIDTH_BYTES_, HUD_HEIGHT_);
		convertIntToChar(levelBatteryFuel, hudText);
        drawString(LIGHT_BATTERY_START_ + HUD_ITEM_WIDTH_BYTES_, HUD_Y_POSITION_, hudText);	

		// Player arrow count.
		hudVideoMemory = cpct_getScreenPtr(CPCT_VMEM_START, ARROW_START_, HUD_Y_POSITION_);
		cpct_drawSprite(G_hud_arrow, hudVideoMemory, HUD_ITEM_WIDTH_BYTES_, HUD_HEIGHT_);
		convertIntToChar(levelArrowCount, hudText);
        drawString(ARROW_START_ + HUD_ITEM_WIDTH_BYTES_, HUD_Y_POSITION_, hudText);

		// Player meat count (for gesture level).
		if (currentLevel == GESTURE_LEVEL)
		{
			hudVideoMemory = cpct_getScreenPtr(CPCT_VMEM_START, MEAT_START_, HUD_Y_POSITION_);
			cpct_drawSprite(G_hud_meat, hudVideoMemory, HUD_ITEM_WIDTH_BYTES_, HUD_HEIGHT_);
			convertIntToChar(player.meat, hudText);
			drawString(MEAT_START_ + HUD_ITEM_WIDTH_BYTES_, HUD_Y_POSITION_, hudText);
		}
		else if (currentLevel == BOSS_LEVEL)
		{
			hudVideoMemory = cpct_getScreenPtr(CPCT_VMEM_START, MEAT_START_, HUD_Y_POSITION_);
			cpct_drawSprite(G_hud_dragon, hudVideoMemory, HUD_ITEM_WIDTH_BYTES_, HUD_HEIGHT_);
			convertIntToChar(finalBossHealth, hudText);
			drawString(MEAT_START_ + HUD_ITEM_WIDTH_BYTES_, HUD_Y_POSITION_, hudText);
		}
		
		if (INVENTORY_LINE_1 & INVENTORY_ITEM_KEY_RED)
		{
			hudVideoMemory = cpct_getScreenPtr(CPCT_VMEM_START, RED_KEY_START_, HUD_Y_POSITION_);
			cpct_drawSprite(G_hud_key_red, hudVideoMemory, HUD_ITEM_THIN_WIDTH_BYTES_, HUD_HEIGHT_);
		}

		if (INVENTORY_LINE_1 & INVENTORY_ITEM_KEY_GREEN)
		{
			hudVideoMemory = cpct_getScreenPtr(CPCT_VMEM_START, RED_KEY_START_ + 2, HUD_Y_POSITION_);
			cpct_drawSprite(G_hud_key_green, hudVideoMemory, HUD_ITEM_THIN_WIDTH_BYTES_, HUD_HEIGHT_);
		}

		if (INVENTORY_LINE_1 & INVENTORY_ITEM_KEY_YELLOW)
		{
			hudVideoMemory = cpct_getScreenPtr(CPCT_VMEM_START, RED_KEY_START_ + 4, HUD_Y_POSITION_);
			cpct_drawSprite(G_hud_key_yellow, hudVideoMemory, HUD_ITEM_THIN_WIDTH_BYTES_, HUD_HEIGHT_);
		}

		if (INVENTORY_LINE_1 & INVENTORY_ITEM_KEY_CARD_RED)
		{
			hudVideoMemory = cpct_getScreenPtr(CPCT_VMEM_START, RED_KEY_START_ + 6, HUD_Y_POSITION_);
			cpct_drawSprite(G_hud_keycard_red, hudVideoMemory, HUD_ITEM_THIN_WIDTH_BYTES_, HUD_HEIGHT_);
		}

		if (INVENTORY_LINE_1 & INVENTORY_ITEM_KEY_CARD_GREEN)
		{
			hudVideoMemory = cpct_getScreenPtr(CPCT_VMEM_START, RED_KEY_START_ + 8, HUD_Y_POSITION_);
			cpct_drawSprite(G_hud_keycard_green, hudVideoMemory, HUD_ITEM_THIN_WIDTH_BYTES_, HUD_HEIGHT_);
		}

		if (INVENTORY_LINE_1 & INVENTORY_ITEM_KEY_CARD_YELLOW)
		{
			hudVideoMemory = cpct_getScreenPtr(CPCT_VMEM_START, RED_KEY_START_ + 10, HUD_Y_POSITION_);
			cpct_drawSprite(G_hud_keycard_yellow, hudVideoMemory, HUD_ITEM_THIN_WIDTH_BYTES_, HUD_HEIGHT_);
		}

		if (INVENTORY_LINE_1 & INVENTORY_ITEM_GLOVE)
		{
			hudVideoMemory = cpct_getScreenPtr(CPCT_VMEM_START, RED_KEY_START_ + 12, HUD_Y_POSITION_);
			cpct_drawSprite(G_hud_glove, hudVideoMemory, HUD_ITEM_THIN_WIDTH_BYTES_, HUD_HEIGHT_);
		}

		if (INVENTORY_LINE_2 & INVENTORY_ITEM_POTION)
		{
			hudVideoMemory = cpct_getScreenPtr(CPCT_VMEM_START, RED_KEY_START_ + 14, HUD_Y_POSITION_);
			cpct_drawSprite(G_hud_test_tube, hudVideoMemory, HUD_ITEM_THIN_WIDTH_BYTES_, HUD_HEIGHT_);
		}

        hudRedraw = FALSE;
    }
}

void drawEquippedItems(u8* memory)
{
	if (inventoryLeftItem)
	{
		cpct_drawSprite(itemToHudSprite(inventoryLeftItem), memory, HUD_ITEM_WIDTH_BYTES_, HUD_HEIGHT_);
	}

	if (inventoryRightItem)
	{
		cpct_drawSprite(itemToHudSprite(inventoryRightItem), memory + 3, HUD_ITEM_WIDTH_BYTES_, HUD_HEIGHT_);
	}
}

u8* itemToHudSprite(u8 item)
{
	switch (item)
	{
		case ITEM_OIL_LAMP:
			return G_hud_oil_lamp;

		case ITEM_SCUBA:
			return G_hud_scuba_gear;

		case ITEM_ELECTRIC_TORCH:
			return G_hud_torch;

		case ITEM_WOOD_BLOCK:
			return G_hud_wood_block;

		case ITEM_ONE_HANDED_SWORD:
			return G_hud_one_handed_sword;

		case ITEM_TWO_HANDED_SWORD:
			return G_hud_two_handed_sword;

		case ITEM_BOW:
			return G_hud_bow;
	}
}