#include "Map.h"

u8 mapCells[MAP_CELL_COUNT];
u8* mapLinks_;
u8 mapLinkElementCount_;
u8 mapEnemies[MAX_ENEMY_ELEMENT_COUNT];
u16 mapEnemyCount;
u8* mapBossCells;
u8 mapBossCellCount;
u8 mapStaticLightSourceCount;
u8* mapStaticLightSources;

u8 mapRedraw = TRUE;
u8 enemyRedraw = FALSE;

void initMap(u8 mapNumber)
{
    u16 i = 0;

    // The end of the source level array (arraypointer + size - 1) as a u16 pointer.
	void* sourceEnd = cave_map[mapNumber] + cave_map_size[mapNumber] - 1;

    // The end of the destination array (arraypointer + size - 1) as a u16 pointer.
    void* destinationEnd = mapCells + MAP_CELL_COUNT - 1;

    //enable map redraw
    mapRedraw = TRUE;

	// Enemy redraw happens after the map is drawn.
	enemyRedraw = FALSE;

    // Decrunch the crunched level array into the destination map array
#ifndef WINCPCTELERA
    cpct_zx7b_decrunch_s(destinationEnd, sourceEnd);
#else
	// Wincpctelera does not implement cpct_zx7b_decrunch_s() but cpct_zx7b_decrunch() is equivalent.
	cpct_zx7b_decrunch(destinationEnd, sourceEnd);
#endif
    
    // Loading links from the map.
	mapLinks_ = cave_links[mapNumber];
	mapLinkElementCount_ = cave_link_count[mapNumber];

	// Loading enemies from the map.
	mapEnemyCount = cave_enemy_count[mapNumber];

	// Only load them into the array if there are enemies in the map.
	if(mapEnemyCount > 0)
	{
		cpct_memcpy(mapEnemies, cave_enemies[mapNumber], mapEnemyCount);
	}	

	// Loading boss information from the map.
	mapBossCells = cave_boss[mapNumber];
	mapBossCellCount = cave_boss_cell_count[mapNumber];

	// Loading static light source information from the map.
	mapStaticLightSourceCount = cave_static_light_source_count[mapNumber];
	mapStaticLightSources = cave_static_light_sources[mapNumber];
}

void drawMap(u8 xScroll, u8 yScroll)
{
    if(mapRedraw)
    {
        //draw the content of the mapArray (map grid)
        u8 x = 0;
        u8 y = 0;
        u8 cell = 0;

        // Start at 20,0 which is the top right section of a 2x2 map.
        //u16 position = XY_TO_POSITION(20, 0);
        u16 position = XY_TO_POSITION(xScroll, yScroll);

        //position is the element in the map tile array
        for (x = 0; x < SCREEN_TILES_X; ++x)
        {
            for (y = 0; y < SCREEN_TILES_Y; ++y)
            {
                cell = mapCells[position];
                drawCell(x, y, cell);

                ++position;
            }

            // Move to the start of the next column (half the column height in a 2x2 screen map), 2/3 of the column height in a 3x3 map.
            position += 24;
        }

        mapRedraw = FALSE;

		// Set enemies to redraw after map has been redrawn.
		enemyRedraw = TRUE;

		// Set boss to redraw after map has been redrawn.
		bossRedraw = TRUE;
    }
}

void drawCell(u8 x, u8 y, u8 cell)
{
    u8* mapMemory = cpct_getScreenPtr(CPCT_VMEM_START, xPositions[x], yPositions[y]);

	// Only draw a cell if the light flag is set indicating it is visible
	if (cell & CELL_LIGHT_FLAG)
	{
		if (cell & CELL_ITEM_FLAG)
		{
			//draw item (-1 because the graphicset will start at 0 index but the type will start at 1)
			cpct_drawSprite(itemGraphics[(cell & CELL_GFX_RANGE) - 1], mapMemory, TILE_WIDTH_BYTES, TILE_HEIGHT);
		}
		else if (cell & CELL_INTERACTABLE_FLAG)
		{
			if ((cell & CELL_GFX_RANGE) == INTERACTABLE_ENEMY_INDICATOR)
			{
				// Draw the dirt floor sprite that will then be overdrawn by the actual enemy.
				cpct_drawSprite(G_tile_floor_sprite, mapMemory, TILE_WIDTH_BYTES, TILE_HEIGHT);
			}
			else
			{
				//draw interactable (-1 because the graphicset will start at 0 index but the type will start at 1)
				cpct_drawSprite(interactableGraphics[(cell & CELL_GFX_RANGE) - 1], mapMemory, TILE_WIDTH_BYTES, TILE_HEIGHT);
			}
		}
		else if (cell & CELL_GFX_RANGE) // Need to exclude the light flag.
		{
			//draw tile (-1 because the graphicset will start at 0 index but the type will start at 1)
			cpct_drawSprite(tileGraphics[(cell & CELL_GFX_RANGE) - 1], mapMemory, TILE_WIDTH_BYTES, TILE_HEIGHT);
		}
		else
		{
			// Draw the appropriate floor sprite.
			cpct_drawSprite(G_tile_floor_sprite, mapMemory, TILE_WIDTH_BYTES, TILE_HEIGHT);
		}
	}
	else
	{
		// Draw black on the screen for the dark cells.
		cpct_drawSolidBox(mapMemory, cpct_px2byteM0(5, 5), TILE_WIDTH_BYTES, TILE_HEIGHT);
	}    
}

void drawBlankCell(u8 x, u8 y)
{
	u8* mapMemory = cpct_getScreenPtr(CPCT_VMEM_START, xPositions[x], yPositions[y]);
    
	// Draw the floor sprite.
    cpct_drawSprite(G_tile_floor_sprite, mapMemory, TILE_WIDTH_BYTES, TILE_HEIGHT);
}

void activateLinkedInteractable(u8 x, u8 y, u8 parent)
{
    u8 i = 0;

    for(i=0; i<mapLinkElementCount_; i+=4)
    {
        if(mapLinks_[i] == x && mapLinks_[i+1] == y)
        {
            activeInteractable(mapLinks_[i+2], mapLinks_[i+3], parent);
            return;
        }
    }
}

void activeInteractable(u8 x, u8 y, u8 parent)
{
	u8 cell = MAPA(x, y);

    switch(cell & CELL_GFX_RANGE)
    {
		case INTERACTABLE_ELECTRIC_DOOR_RED:
		case INTERACTABLE_ELECTRIC_DOOR_GREEN:
		case INTERACTABLE_ELECTRIC_DOOR_YELLOW:
        case INTERACTABLE_LASER_GATE:
        {
			// Reset the linked cell to blank (removes the interactable) but only visible if it is currently visible.
			if (cell & CELL_LIGHT_FLAG)
			{
				mapCells[XY_TO_POSITION(x, y)] = postMoveCell | CELL_LIGHT_FLAG;
			}
			else
			{
				mapCells[XY_TO_POSITION(x, y)] = postMoveCell;
			}

			// Note: I did intend to do a redraw of the cell here but it looks like a map redraw happens after anyway. 
        }
        break;

		// Off teleporter set to on (I'm not going to bother with the opposite unless I need it).
		// TODO implement the opposite if it becomes desirable such as light attracted enemies.
		case INTERACTABLE_TELEPORTER_OFF:
		{
			mapCells[XY_TO_POSITION(x, y)] = INTERACTABLE_TELEPORTER_ON | CELL_INTERACTABLE_FLAG;

			// Apply the light flag if it is currently set on the cell.
			if (cell & CELL_LIGHT_FLAG)
			{
				mapCells[XY_TO_POSITION(x, y)] |= CELL_LIGHT_FLAG;
			}

			mapRedraw = TRUE;
		}
		break;

		case INTERACTABLE_TELEPORTER_ON:
		{
			// Teleporting is only available from other teleporters.
			if (parent == INTERACTABLE_TELEPORTER_ON)
			{
				playerTeleportingX = x;
				playerTeleportingY = y;
			}
		}
		break;
    }
}