#ifndef PLAYER_H
#define PLAYER_H

#include <cpctelera.h>
#include "Cell.h"
#include "Difficulty.h"
#include "Game.h"
#include "GameState.h"
#include "GameTypes.h"
#include "Hud.h"
#include "Interactable.h"
#include "Interrupt.h"
#include "Inventory.h"
#include "Item.h"
#include "Keys.h"
#include "Lighting.h"
#include "Map.h"
#include "Object.h"
#include "Projectile.h"
#include "Tile.h"
#include "sprites/PlayerSprites.h"
#include "sprites/MiscSprites.h"

#define PLAYER_WIDTH 4
#define PLAYER_HEIGHT 16
#define PLAYER_MAX_HEALTH 10

typedef struct Player Player;

struct Player
{
    Object object;
	u8 health;	
	u8 moving : 1;
	u8 cursorDirection : 4;
	u8 spriteDirection : 2;
	u8 blocked : 1;
	u8 meat;
};

//global player object
extern Player player;

//initialises the player object values
void initPlayer(u8 x, u8 y);

//draws the player on the screen as positioned in the player object
void drawPlayer(u8 xScroll, u8 yScroll);

//moves the player position based on keyboard input
void movePlayer();

//runs any logic required for the player function such as interactions and velocity/position
void logicPlayer(u8* xScroll, u8* yScroll);

u8 processCell(u8 x, u8 y, u8 xScroll, u8 yScroll);

// Tests if an interactable is ready for it to be used\activated.
u8 isInteractableReady(u8 cell);

// Draw indicator on the screen showing where the next player move is.
void drawPlayerNextMoveIndicator(u8 xScroll, u8 yScroll);

// Determines where the player is able to move next based on the direction, this updates player.object.nextPosition.
void determinePlayerNextMove(u8 xScroll, u8 yScroll);

// Updates the scroll values which determines which cells in the map grid are displayed on the screen.
void scrollScreen(u8* xScroll, u8* yScroll);

// Updates the scroll values following teleporting.
void teleporterScroll(u8* xScroll, u8* yScroll);

// Updates the player stats based on the item.
void consumeItem(u8 cell);

// Scrolls the scren after a big player movement such as teleportation.
void catchupScroll(u8* xScroll, u8* yScroll, u8* destinationX, u8* destinationY);

// Attempts to attack an enemy at the specified position.
void attackEnemy(u8 x, u8 y, u8 attackType, u8 xScroll, u8 yScroll);

// Attempt to attack a boss cell at the specified position.
void attackBoss(u8 x, u8 y, u8 attackType, u8 xScroll, u8 yScroll);

#endif