#include "Projectile.h"

u8 projectileNextDirection;
u8 activeProjectile;
u8 projectileDirection;
Object projectile;

void drawProjectile(u8 xScroll, u8 yScroll)
{
	if (activeProjectile)
	{
		u8 cell = MAPA(projectile.position.x, projectile.position.y);

        // Make sure the cell is lit before displaying the projectile.
		if (cell & CELL_LIGHT_FLAG)
		{
			// TODO projectile graphics for each direction.
			u8 x = projectile.position.x - xScroll;
			u8 y = projectile.position.y - yScroll;

			// Make sure in range (on screen), when negative it becomes 255 which is out of range.
			if (x < SCREEN_TILES_X && y < SCREEN_TILES_Y)
			{
				u8* mapMemory = cpct_getScreenPtr(CPCT_VMEM_START, xPositions[x], yPositions[y]);

				if(projectile.direction == DIRECTION_RIGHT)
				{
					cpct_drawSprite(G_projectile_arrow_right, mapMemory, TILE_WIDTH_BYTES, TILE_HEIGHT);
				}
				else if(projectile.direction == DIRECTION_LEFT)
				{
					cpct_drawSprite(G_projectile_arrow_left, mapMemory, TILE_WIDTH_BYTES, TILE_HEIGHT);
				}
				else if(projectile.direction == DIRECTION_UP)
				{
					cpct_drawSprite(G_projectile_arrow_up, mapMemory, TILE_WIDTH_BYTES, TILE_HEIGHT);
				}
				else
				{
					// Down
					cpct_drawSprite(G_projectile_arrow_down, mapMemory, TILE_WIDTH_BYTES, TILE_HEIGHT);
				}				
			}
		}
	}
}

void logicProjectile(u8 xScroll, u8 yScroll)
{
	// Make sure it is the projectile's turn in the turn sequence.
	if (gameTurnState == PROJECTILE_TURN)
	{
		// Only run the logic if the projectile is active (has spawned).
		if (activeProjectile && gameTurnState == PROJECTILE_TURN)
		{
			u8 cell = 0;
			u8 nextCell = 0;

			// Determine the next position for the projectile.
			switch (projectile.direction)
			{
				case DIRECTION_RIGHT:
					projectile.nextPosition.x = projectile.position.x + 1;
					projectile.nextPosition.y = projectile.position.y;
				break;

				case DIRECTION_LEFT:
					projectile.nextPosition.x = projectile.position.x - 1;
					projectile.nextPosition.y = projectile.position.y;
				break;

				case DIRECTION_UP:
					projectile.nextPosition.x = projectile.position.x;
					projectile.nextPosition.y = projectile.position.y - 1;
				break;

				case DIRECTION_DOWN:
					projectile.nextPosition.x = projectile.position.x;
					projectile.nextPosition.y = projectile.position.y + 1;
				break;
			}

			cell = MAPA(projectile.position.x, projectile.position.y);

			// Make sure the next position is in range.
			if (projectile.nextPosition.x >= MAP_TILES_X || projectile.nextPosition.y >= MAP_TILES_Y)
			{
				// Set the next cell to solid but not an enemy so it clears up the projectile.
				nextCell = 1;
			}
			else
			{
				nextCell = MAPA(projectile.nextPosition.x, projectile.nextPosition.y);
			}

			// Check that the next position empty (exclude the light flag) or not.
			// 127 is the first 7 bits 0b01111111.
			if (nextCell & 127)
			{
				// 127 is bottom 7 bits 0b01111111. 
				// 95 is 0b1011111 is an interactable enemy indicator.
				if ((nextCell & 127) == 95)
				{
					// Attack the enemy at next position.
					attackEnemy(projectile.nextPosition.x, projectile.nextPosition.y, RANGED_ATTACK, xScroll, yScroll);
				}
				else if ((nextCell & 127) == 94)
				{
					// 127 is bottom 7 bits 0b01111111. 
					// 94 is 0b1011110 is an interactable boss indicator.

					// Attack the boss at next position.
					attackBoss(projectile.nextPosition.x, projectile.nextPosition.y, RANGED_ATTACK, xScroll, yScroll);
				}

				// Cell not empty so set active to false destroying the projectile.
				activeProjectile = FALSE;

				// Clear the cell that the projectile is currently at as it no longer exists.
				drawCell(projectile.position.x - xScroll, projectile.position.y - yScroll, cell);
			}
			else
			{
				// Clear the previous cell after the projectile moves.
				drawCell(projectile.position.x - xScroll, projectile.position.y - yScroll, cell);

				// Cell is empty so update the projectile.
				projectile.position.x = projectile.nextPosition.x;
				projectile.position.y = projectile.nextPosition.y;
			}
		}

		// Move onto the enemy turn after the projectile logic has run.
		gameTurnState = ENEMY_TURN;
	}
}